﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;

using Model;
using View;
using System.Threading;
using System.Diagnostics;

namespace Controller
{
    static class MouseHandler
    {
        private static MouseState lastMouseState;          // this state allows for detection of a single click (not click and hold)
        private static MouseState currentMouseState;
        private static TurnState turnState = TurnState.Wait;

        public static void updateMouse(Graph graph)
        {
            lastMouseState = currentMouseState;
            currentMouseState = Mouse.GetState();

            // valid coordinates for the mouse (ie. must be inside the game window)
            bool validX = currentMouseState.X < GameState.SCREEN_WIDTH && currentMouseState.X > 0;
            bool validY = currentMouseState.Y < GameState.SCREEN_HEIGHT && currentMouseState.Y > 0;

            // if unit is selected when right-clicked, deselect it
            if (lastMouseState.RightButton == ButtonState.Released
                && currentMouseState.RightButton == ButtonState.Pressed
                && validX && validY && GameState.playableUnitSelected)
            {
                    setSelectedUnit(null, false);
            }

            // checks for single mouse left click inside the game window
            if (lastMouseState.LeftButton == ButtonState.Released
                && currentMouseState.LeftButton == ButtonState.Pressed
                && validX && validY)
            {
                // do not react to mouse input if animation is on screen
                if (GameState.isAnimating)
                {
                    return;
                }

                Vector2 mouseClickCoordinates = new Vector2(currentMouseState.X, currentMouseState.Y);   // mouse click coordinates
                Node nodeClickedOn = graph.getNode(mouseClickCoordinates);

                #region Check if a unit is clicked

                // if player clicks after unit is already selected ...
                if (GameState.playableUnitSelected)
                {
                    // determine if player has clicked on a menu button; if so, change turnState
                    if (getMenuButtonClicked(mouseClickCoordinates) != null)
                    {
                        MenuButton menuButton = getMenuButtonClicked(mouseClickCoordinates);
                        buttonAction(menuButton);
                        return;
                    }



                    // if user clicks on a valid end node, move to it
                    if (turnState == TurnState.Move)
                    {
                        Game.Instance.Tick();

                        // set variables for path finding
                        Node startNode = graph.getNode(GameState.selectedUnit.Position);
                        Node endNode = graph.getNode(mouseClickCoordinates);
                        LinkedList<Node> path = GameFunction.pathFinder(graph, GameState.selectedUnit, startNode, endNode);

                        // if path finder returns a non-null path, then end node is valid
                        if (path != null)
                        {
                            updateUnitPosition(graph, mouseClickCoordinates, path);
                        }

                        turnState = TurnState.Wait;
                    }

                    if(turnState==TurnState.Attack) //if a unit is clicked after attack is clicked
                    {
                        Unit unit = getPlayableUnitOnNodeClicked(graph.getNode(mouseClickCoordinates), mouseClickCoordinates, GameState.currentPlayer);
                        if (unit != null)
                    {
                        setAttackedUnit(unit, true);
                            attackAnimation(graph, 0);
                        
                        //GameState.dropDownMenuOpen = (true);

                        
                    }
                    }
                }

                // if player clicks when a unit is not selected ...
                else
                {
                    // if there is a playable, player-owned unit on position clicked, set selected unit status
                    Unit unit = getPlayableUnitOnNodeClicked(graph.getNode(mouseClickCoordinates), mouseClickCoordinates, GameState.currentPlayer);
                    if (unit != null)
                    {
                        setSelectedUnit(unit, true);
                        GameFunction.setMovableNodes(graph, unit);
                        GameState.dropDownMenuOpen = (true);
                        GameState.beforeMove = (true);
                    }
                }

                #endregion
            }
        }

        // sets selection of unit state inside GameFunction and the unit itself
        private static void setSelectedUnit(Unit unit, bool selected)
        {
            GameState.playableUnitSelected = selected;
            GameState.selectedUnit = unit;
        }

        //sets selection of unit to attack inside GameFunction 
        private static void setAttackedUnit(Unit unit, bool selected)
        {
            GameState.unitToAttack = unit;
        }

        // if playable unit exists where user clicked, return it; else, return null
        private static Unit getPlayableUnitOnNodeClicked(Node clickedNode, Vector2 positionClicked, Player currentPlayer)
        {
            for (int i = 0; i < currentPlayer.getNumOfUnits(); i++)
            {
                Unit unit = currentPlayer.getUnits().ElementAt(i);
                int unitX = unit.Position.Item1;
                int unitY = unit.Position.Item2;
                int clickedX = (int)Math.Floor(positionClicked.X / 32);
                int clickedY = (int)Math.Floor(positionClicked.Y / 32);
                if (unitX == clickedX && unitY == clickedY)
                {
                    return unit;
                }
            }
            return null;
        }

        // update the unit's position to the clicked position
        private static void updateUnitPosition(Graph graph, Vector2 position, LinkedList<Node> path)
        {
            Unit unit = GameState.selectedUnit;
            GameState.isAnimating = (true);
            // updates the unit's position to each node in the path (node by node)
            foreach (Node node in path)
            {
                animateUnitPosition(graph, unit, node);
                unit.Position = (new Tuple<int, int>(node.getPositionX(), node.getPositionY()));
                node.unitOnNode = (unit);
                GameState.dropDownMenuOpen = (true);
                
            }
            GameState.isAnimating = (false);
            GameState.beforeMove = (false);
        }

        private static void attackAnimation(Graph graph, int direction)
        {
            Unit unit = GameState.selectedUnit;
            GameState.isAnimating = true;
            float originalLocationX = unit.PixelCoordinates.X;
            float originalLocationY = unit.PixelCoordinates.Y;
            #region Attack Right
            if (direction == 0) //attack right
            {
                for (float i = originalLocationX; i <= originalLocationX + 8; i++)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X + 1, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }

                for (float i = unit.PixelCoordinates.X; i >= originalLocationX; i--)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X - 1, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }
            }
            #endregion
            #region Attack Left
            else if (direction == 1) //attack left
            {
                for (float i = originalLocationX; i >= originalLocationX - 8; i--)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X + 1, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }

                for (float i = unit.PixelCoordinates.X; i <= originalLocationX; i++)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X - 1, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }
            }
            #endregion
            #region Attack Up
            else if (direction == 2) //attack up
            {
                for (float i = originalLocationY; i >= originalLocationY - 8; i--)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X , unit.PixelCoordinates.Y +1);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }

                for (float i = unit.PixelCoordinates.X; i <= originalLocationX; i++)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X , unit.PixelCoordinates.Y - 1);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }
            }
            #endregion
            #region Attack Down
            else if (direction == 3) //attack down
            {
                for (float i = originalLocationY; i >= originalLocationY - 8; i--)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y + 1);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }

                for (float i = unit.PixelCoordinates.X; i <= originalLocationX; i++)
                {
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y - 1);
                    Game.Instance.Tick();
                    Thread.Sleep(10);
                }
            }
            #endregion
            GameState.isAnimating = false;
        }

        // Moves unit to move from to different nodes
        private static void animateUnitPosition(Graph graph, Unit unit, Node node)
        {
            int nodePixelX = node.getPositionX() * 32;
            int nodePixelY = node.getPositionY() * 32;
            int pixelDifference = 4;
            graph.getNode(unit.Position).unitOnNode = (null);

            while (unit.PixelCoordinates.X != nodePixelX)
            {
                if (unit.PixelCoordinates.X < nodePixelX)
                {
                    unit.animate(2);
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X + pixelDifference, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(40);
                }
                else
                {
                    unit.animate(1);
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X - pixelDifference, unit.PixelCoordinates.Y);
                    Game.Instance.Tick();
                    Thread.Sleep(40);
                }
            }

            while (unit.PixelCoordinates.Y != nodePixelY)
            {
                if (unit.PixelCoordinates.Y < nodePixelY)
                {
                    unit.animate(0);
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y + pixelDifference);
                    Game.Instance.Tick();
                    Thread.Sleep(40);
                }
                else
                {
                    unit.animate(3);
                    unit.PixelCoordinates = new Vector2(unit.PixelCoordinates.X, unit.PixelCoordinates.Y - pixelDifference);
                    Game.Instance.Tick();
                    Thread.Sleep(40);
                }
            }
        }

        // returns a menu button that was clicked; if no menu button was clicked, returns null
        private static MenuButton getMenuButtonClicked(Vector2 mouseCoordinates)
        {
            Unit unit = GameState.selectedUnit;
            MenuButton[] menuButtons = unit.getMenuButtons();
            int clickX = (int) mouseCoordinates.X;
            int clickY = (int) mouseCoordinates.Y;

            for (int i=0; i<menuButtons.Length; i++)
            {
                int buttonX = (int) menuButtons[i].getPixelCoordinates().X;
                int buttonY = (int) menuButtons[i].getPixelCoordinates().Y;
                if (buttonX <= clickX && clickX < buttonX+128 && buttonY <= clickY && clickY < buttonY+32)
                {
                    if (GameState.dropDownMenuOpen)
                    {
                    return menuButtons[i];
                    }
                }
            }
            return null;
        }

        // sets which menu option is selected
        private static void buttonAction(MenuButton button)
        {
            // take action corresponding to which button was clicked
            switch (button.getButtonType())
            {
                
                case MenuButtonType.Attack:         // if attackMenu clicked
                    turnState = TurnState.AttackMenu;
                    GameState.dropDownMenuOpen = (false);    // close the dropdownmenu when selecting who to attack
                    button.setActive(false);
                    break;
                case MenuButtonType.AttackConfirm:         // if confirm attack clicked
                    turnState = TurnState.Attack;
                    button.setActive(false);
                    Unit unit = GameState.selectedUnit;
                    MenuButton moveButtons = unit.getMenuButtonAt(1);
                    break;
                case MenuButtonType.Move:           // if moved is clicked
                    turnState = TurnState.Move;
                    GameState.dropDownMenuOpen = (false);   // close the dropdownmenu when selecting where to move
                    button.setActive(false);
                    break;
                case MenuButtonType.Items:          // if item is clicked
                    turnState = TurnState.Items;
                    GameState.dropDownMenuOpen = (false);
                    break;
                case MenuButtonType.Wait:           // if wait is clicked
                    turnState = TurnState.Wait;
                    GameState.dropDownMenuOpen = (false);
                    GameState.playableUnitSelected = false;
                    button.setActive(false);
                    break;

                default:
                    break;
            }
        }
    }
}
