﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

using View;
using System.Diagnostics;

namespace Model
{
    class Warrior : Unit
    {
        public bool Alive { get; set; }
        public int Hp { get; set; }
        public int Str { get; set; }
        public int Int { get; set; }
        public int Skill { get; set; }
        public int Speed { get; set; }
        public int Def { get; set; }
        public int Res { get; set; }
        public int Level { get; set; }  
        private readonly int movability = 4;    // all warriors will have 5 movability
        private Weapon[] equipableWeapons;
        private Weapon equippedWeapon;
        private Texture2D spriteImage;
        private Vector2 pixelCoordinates;
        private Tuple<int, int> position;
        private MenuButton[] menuButtons = new MenuButton[4];
        private Texture2D charInfo;
        private int currentFrame; //the current frame the sprite is on
        private int player; // which player this unit belongs to

        public Warrior(Texture2D spriteImage, MenuButton attackButton, MenuButton moveButton,
            MenuButton itemButton, MenuButton waitButton, Texture2D charInfo, Vector2 coordinates, int player)
        {
            this.spriteImage = spriteImage;
            menuButtons[0] = attackButton;
            menuButtons[1] = moveButton;
            menuButtons[2] = itemButton;
            menuButtons[3] = waitButton;
            this.charInfo = charInfo;
            this.player = player;
            pixelCoordinates = coordinates;
            int positionX = (int)Math.Round(coordinates.X / 32);
            int positionY = (int)Math.Round(coordinates.Y / 32);
            position = new Tuple<int, int>(positionX, positionY);
            currentFrame = 1;
            setMenuButtonCoordinates(pixelCoordinates);
            setInitialStats(); //sets initial warrior stats
        }

        //sets initial unit stats
        public void setInitialStats()
        {
            Alive = true;
            Level = 1;
            Hp = 10;
            Str = 9;
            Int = 1;
            Skill = 6;
            Speed = 8;
            Def = 8;
            Res = 4;
        }

        public int getMovability()
        {
            return movability;
        }

        //returns each stat as an index for incrementing purposes
        public int getStats(int i) 
        {
            int[] allStats = new int[7];
            allStats[0] = Level;
            allStats[1] = Str;
            allStats[2] = Int;
            allStats[3] = Skill;
            allStats[4] = Speed;
            allStats[5] = Def;
            allStats[6] = Res;
            return allStats[i];
        }

        public Weapon[] getEquipableWeapons()
        {
            return equipableWeapons;
        }

        public Weapon getEquippedWeapon()
        {
            return equippedWeapon;
        }

        public UnitType getClass()
        {
            return UnitType.Warrior;
        }

        public Texture2D getSpriteImage()
        {
            return spriteImage;
        }

        public Texture2D getButtonImage(MenuButtonType buttonType)
        {
            switch (buttonType)
            {
                case MenuButtonType.Attack: // if attack clicked
                    return menuButtons[0].getImage();
                case MenuButtonType.Items: // if moved is clicked
                    return menuButtons[2].getImage();
                case MenuButtonType.Move: // if item is clicked
                    return menuButtons[1].getImage();
                case MenuButtonType.Wait: // if wait is clicked
                    return menuButtons[3].getImage();

                default:
                    return null;
            }
        }

        // indicates whether a button has already been previously selected or not
        public bool isButtonActive(MenuButtonType buttonType)
        {
            switch (buttonType)
            {
                case MenuButtonType.Attack:
                    return menuButtons[0].getActive();
                case MenuButtonType.Items:
                    return menuButtons[2].getActive();
                case MenuButtonType.Move:
                    return menuButtons[1].getActive();
                case MenuButtonType.Wait:
                    return menuButtons[3].getActive();

                default:
                    return false;
            }
        }

        public Texture2D getCharInfo()
        {
            return charInfo;
        }

        public Tuple<int, int> Position
            {
            get
                {
                    return position;
                }
            set
                {
                    position = value;
                    pixelCoordinates = new Vector2(value.Item1 * 32, value.Item2 * 32);
                }
            }

        public Vector2 PixelCoordinates
        {
            get
                {
                    return pixelCoordinates;
                }
            set
                {
                    pixelCoordinates = value;
                    int positionX = (int)Math.Round(PixelCoordinates.X / 32);
                    int positionY = (int)Math.Round(PixelCoordinates.Y / 32);
                    position = new Tuple<int, int>(positionX, positionY);
                }
        }

        public void setEquippedWeapon(Weapon w)
        {
            equippedWeapon = w;
        }

        public MenuButton[] getMenuButtons()
        {
            return menuButtons;
        }

        public MenuButton getMenuButtonAt(int i)
        {
            return menuButtons[i];
        }
        
        // updates menu button positions
        public void setMenuButtonCoordinates(Vector2 pixelCoordinates)
        {
            int increment = 0;
            foreach (MenuButton button in menuButtons)
            {
                if (button.getActive())
                {
                    button.setPixelCoordinates((int)(pixelCoordinates.X + 32), (int)(pixelCoordinates.Y + (increment * 32)));
                    increment++;
                }
            }
        }

        public Rectangle getCurrentFrame() //return current frame the sprite is on
        {
            Rectangle screenBounds = new Rectangle(currentFrame * 32, 0, 32, 32);
            return screenBounds;
        }

        // animates the sprite by switching to the correct frame sequences
        public void animate(int direction) 
        {

            #region Walking Down
            if (direction == 0) // for going down
            {
                if (currentFrame > 2) // if unit isnt already going down, set the sprite to default facing downwards
                {
                    currentFrame = 1;
                }
                else
                {
                    if (currentFrame == 2) // if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 0;
                    }
                    else
                    {
                        currentFrame++; // increment through walk cycle
                    }
                }
            }
            #endregion

            #region Walking Left
            else if(direction == 1) // for going left
            {
                if ((currentFrame < 3) || (currentFrame > 5)) // if unit isnt already going left, set the sprite to default facing left
                {
                    currentFrame = 4;
                }
                else
                {
                    if (currentFrame == 5) // if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 3;
                    }
                    else
                    {
                        currentFrame++; //increment walk cycle
                    }
                }
            }
            #endregion

            #region Walking Right
            else if (direction == 2) // for going right
            {
                if ((currentFrame < 6) || (currentFrame > 8)) // if unit isnt already going right, set the sprite to default facing right
                {
                    currentFrame = 7;//increment walk cycle
                }
                else
                {
                    if (currentFrame == 8)// if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 6;
                    }
                    else
                    {
                        currentFrame++;
                    }
                }
            }
            #endregion

            #region Walking Up
            else if (direction == 3) // for going up
            {
                if (currentFrame < 9) // if unit isnt already going up, set the sprite to default facing upwards
                {
                    currentFrame = 10;//increment walk cycle
                }
                else
                {
                    if (currentFrame == 11)// if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 9;//increment walk cycle
                    }
                    else
                    {
                        currentFrame++;
                    }
                }
            }
            #endregion
        }
    }
}
